@@ -6,6 +6,7 @@ from emora_stdm.state_transition_dialogue_manager.knowledge_base import Knowledg
 from time import time
 import dill
 from pathos.multiprocessing import ProcessingPool as Pool
+import traceback
 
 def precache(transition_datas):
     for tran_datas in transition_datas:
@@ -16,7 +17,7 @@ def precache(transition_datas):
 class CompositeDialogueFlow:
 
     def __init__(self, initial_state, system_error_state, user_error_state,
-                 initial_speaker = None, macros=None, kb=None):
+                 initial_speaker = DialogueFlow.Speaker.SYSTEM, macros=None, kb=None):
         if isinstance(system_error_state, str):
             system_error_state = ('SYSTEM', system_error_state)
         if isinstance(user_error_state, str):
@@ -36,7 +37,7 @@ class CompositeDialogueFlow:
         :return: None
         """
         while True:
-            if self._controller.speaker() == DialogueFlow.Speaker.SYSTEM:
+            if self.controller().speaker() == DialogueFlow.Speaker.SYSTEM:
                 print("S:", self.system_turn(debugging=debugging))
             else:
                 user_input = input("U: ")
@@ -50,21 +51,21 @@ class CompositeDialogueFlow:
         """
         visited = {self._controller.state()}
         responses = []
-        while self._controller.speaker() is DialogueFlow.Speaker.SYSTEM:
+        while self.controller().speaker() is DialogueFlow.Speaker.SYSTEM:
             try:
-                response, next_state = self._controller.system_transition(self._controller.state(), debugging=debugging)
-                self._controller.take_transition(next_state)
+                response, next_state = self.controller().system_transition(self.controller().state(), debugging=debugging)
+                assert next_state is not None
+                self.controller().set_state(next_state)
             except Exception as e:
-                print('Error in CompositeDialogueFlow. Component: {}  State: {}'.format(self._controller_name, self._controller.state()))
-                print(e)
+                print('Error in CompositeDialogueFlow. Component: {}  State: {}'.format(self.controller(), self.controller().state()))
+                traceback.print_exc(file=sys.stdout)
                 response, next_state = '', self._system_error_state
                 visited = visited - {next_state}
             if isinstance(next_state, tuple):
                 self.set_control(*next_state)
             responses.append(response)
-            if next_state in visited and self._controller._speaker is DialogueFlow.Speaker.SYSTEM:
-                self._controller.change_speaker()
-                break
+            if next_state in visited or not self.state_settings(*self.state()).system_multi_hop:
+                self.controller().set_speaker(DialogueFlow.Speaker.USER)
             visited.add(next_state)
         return  ' '.join(responses)
 
@@ -77,43 +78,49 @@ class CompositeDialogueFlow:
         :return: None
         """
         try:
-            self._controller.state_update(natural_language, debugging=debugging)
-            next_state = self._controller.state()
+            self.controller().apply_update_rules(natural_language, debugging=debugging)
+            next_state = self.controller().state()
         except Exception as e:
-            print('Error in CompositeDialogueFlow. Component: {}  State: {}'.format(self._controller_name, self._controller.state()))
-            print(e)
+            print('Error in CompositeDialogueFlow. Component: {}  State: {}'.format(self._controller_name, self.controller().state()))
+            traceback.print_exc(file=sys.stdout)
             next_state = self._user_error_state
-        visited = {self._controller.state()}
-        while self._controller.speaker() is DialogueFlow.Speaker.USER:
+        visited = {self.controller().state()}
+        while self.controller().speaker() is DialogueFlow.Speaker.USER:
             try:
-                next_state = self._controller.user_transition(natural_language, self._controller.state(), debugging=debugging)
-                self._controller.take_transition(next_state)
+                next_state = self.controller().user_transition(natural_language, self.controller().state(), debugging=debugging)
+                assert next_state is not None
+                self.controller().set_state(next_state)
             except Exception as e:
-                print('Error in CompositeDialogueFlow. Component: {}  State: {}'.format(self._controller_name, self._controller.state()))
-                print(e)
+                print('Error in CompositeDialogueFlow. Component: {}  State: {}'.format(self._controller_name, self.controller().state()))
+                traceback.print_exc(file=sys.stdout)
                 next_state = self._user_error_state
             next_state = module_state(next_state)
             if isinstance(next_state, tuple):
                 self.set_control(*next_state)
-                if self._controller.speaker() is DialogueFlow.Speaker.USER:
-                    self._controller.state_update(natural_language, debugging=debugging)
-            if next_state in visited and self._controller._speaker is DialogueFlow.Speaker.USER:
-                self._controller.change_speaker()
-                break
+                if self.state_settings(*self.state()).user_multi_hop:
+                    self.controller().apply_update_rules(natural_language, debugging=debugging)
+            if next_state in visited or not self.state_settings(*self.state()).user_multi_hop:
+                self.controller().set_speaker(DialogueFlow.Speaker.SYSTEM)
             visited.add(next_state)
         next_state = module_state(next_state)
         if isinstance(next_state, tuple):
             self.set_control(*next_state)
-            if self._controller.speaker() is DialogueFlow.Speaker.USER:
-                self._controller.state_update(natural_language, debugging=debugging)
+            if self.controller().speaker() is DialogueFlow.Speaker.USER:
+                self.controller().apply_update_rules(natural_language, debugging=debugging)
 
 
     def set_control(self, namespace, state):
         state = module_state(state)
-        speaker = self._controller.speaker()
+        speaker = self.controller().speaker()
+        old_state = self.controller().state()
+        self.component(namespace).set_state(old_state)
         self.set_controller(namespace)
-        self._controller.set_speaker(speaker)
-        self._controller.set_state(state)
+        self.controller().set_speaker(speaker)
+        if speaker == DialogueFlow.Speaker.USER and not self.controller().state_settings(state).user_multi_hop:
+            self.controller().change_speaker()
+        elif speaker == DialogueFlow.Speaker.SYSTEM and not self.controller().state_settings(state).system_multi_hop:
+            self.controller().change_speaker()
+        self.controller().set_state(state)
 
     def precache_transitions(self, process_num=1):
         start = time()
@@ -172,25 +179,43 @@ class CompositeDialogueFlow:
 
     def add_component(self, component, namespace):
         self._components[namespace] = component
-        component.set_is_module()
+        component.set_is_module(self)
+        component.set_namespace(namespace)
+        component.set_gates(self.component('SYSTEM').gates())
 
     def component(self, namespace):
         return self._components[namespace]
 
     def set_state(self, state):
         state = module_state(state)
-        self._controller.set_state(state)
+        if isinstance(state, tuple):
+            if self.component(state[0]) != self.controller():
+                self.component(state[0]).set_state(self.controller().state())  # so __system_state__ is set properly
+                self.set_controller(state[0])
+                state = state[1]
+        self.controller().set_state(state)
 
     def set_controller(self, controller_name):
         old_controller_vars = self._controller.vars()
         if self._controller_name != controller_name:
-            del old_controller_vars["__state__"]
+            del old_controller_vars['__state__']
         self._controller = self.component(controller_name)
         self._controller_name = controller_name
         new_controller_vars = self._controller.vars()
         new_controller_vars.update(old_controller_vars)
         self._controller.set_vars(new_controller_vars)
 
+    def transition_natex(self, namespace, source, target, speaker):
+        source, target = module_source_target(source, target)
+        if isinstance(source, tuple):
+            source = source[1]
+        if isinstance(target, tuple):
+            target = target[1]
+        return self.component(namespace).transition_natex(source, target, speaker)
+
+    def state_settings(self, namespace, state):
+        return self.component(namespace).state_settings(state)
+
     def set_vars(self, vars):
         self._controller.set_vars(vars)
 
