@@ -362,8 +362,8 @@ class DialogueFlow:
                 generation = natex.generate(vars=vars, macros=self._macros, debugging=debugging)
             except Exception as e:
                 print()
-                print(e)
                 print('Transition {}: {} failed'.format(str(transition), natex))
+                traceback.print_exc(file=sys.stdout)
                 print()
                 generation = None
             source, target, speaker = transition
@@ -380,11 +380,14 @@ class DialogueFlow:
                 transition = (target.split('->')[0], target.split('->')[1], speaker)
                 try:
                     appended_generation = self.transition_natex(*transition).generate(vars=vars, macros=self._macros, debugging=debugging)
-                    generation = generation + ' ' + appended_generation
+                    if appended_generation is None:
+                        generation = None
+                    else:
+                        generation = generation + ' ' + appended_generation
                 except Exception as e:
                     print()
-                    print(e)
                     print('Transition {}: {} failed'.format(str(transition), natex))
+                    traceback.print_exc(file=sys.stdout)
                     print()
                     generation = None
             elif isinstance(transition[1], tuple) and '->' in transition[1][1]:
@@ -396,8 +399,8 @@ class DialogueFlow:
                     generation = generation + ' ' + appended_generation
                 except Exception as e:
                     print()
-                    print(e)
                     print('Transition {}: {} failed'.format(str(transition), natex))
+                    traceback.print_exc(file=sys.stdout)
                     print()
                     generation = None
             source, target, speaker = transition
@@ -518,8 +521,8 @@ class DialogueFlow:
                 match = natex.match(natural_language, vars, self._macros, ngrams, debugging)
             except Exception as e:
                 print()
-                print(e)
                 print('Transition {}: {} failed'.format(str(transition), natex))
+                traceback.print_exc(file=sys.stdout)
                 print()
                 match = None
             source, target, speaker = transition
